package com.denizenscript.clientizen.objects;

import com.denizenscript.clientizen.util.Utilities;
import com.denizenscript.denizencore.objects.*;
import com.denizenscript.denizencore.objects.properties.PropertyParser;
import com.denizenscript.denizencore.tags.Attribute;
import com.denizenscript.denizencore.tags.ObjectTagProcessor;
import com.denizenscript.denizencore.tags.TagContext;
import com.denizenscript.denizencore.utilities.debugging.Debug;
import net.minecraft.class_1799;
import net.minecraft.class_1935;
import net.minecraft.class_2960;
import net.minecraft.class_7923;

public class ItemTag implements ObjectTag, Adjustable {

    // <--[ObjectType]
    // @name ItemTag
    // @prefix i
    // @base ElementTag
    // @implements PropertyHolderObject
    // @ExampleTagBase client.self_entity.item_in_hand
    // @ExampleValues <client.self_entity.item_in_hand>,stick,iron_sword
    // @ExampleForReturns
    // - narrate "The item is %VALUE%"
    // @format
    // The identity format for items is the basic material type name. Other data is specified in properties.
    // For example, 'i@stick'.
    //
    // @description
    // An ItemTag represents a holdable item generically.
    //
    // ItemTags do NOT remember where they came from. If you read an item from somewhere, changing it
    // does not change the original item. You must set it back in.
    //
    // Find a list of valid materials at <@link url https://hub.spigotmc.org/javadocs/spigot/org/bukkit/Material.html>
    // Note that some materials on that list are exclusively for use with blocks, and cannot be held as items.
    //
    // -->

    final class_1799 itemStack;
    final class_2960 identifier;
    public String script; // Compact with server-side item scripts

    public ItemTag(class_1799 itemStack) {
        this.itemStack = itemStack;
        this.identifier = class_7923.field_41178.method_10221(itemStack.method_7909());
    }

    public ItemTag(class_1935 convertible) {
        this(convertible.method_8389().method_7854());
    }

    public ItemTag(MaterialTag material) {
        this(material.item != null ? material.item : material.state.method_26204());
    }

    @Fetchable("i")
    public static ItemTag valueOf(String string, TagContext context) {
        if (string.startsWith("i@")) {
            string = string.substring("i@".length());
        }
        if (ObjectFetcher.isObjectWithProperties(string)) {
            return ObjectFetcher.getObjectFromWithProperties(ClientizenObjectRegistry.TYPE_ITEM, string, context);
        }
        MaterialTag material = MaterialTag.valueOf(string, context);
        if (material == null) {
            if (context == null || context.showErrors()) {
                Debug.echoError("valueOf ItemTag returning null: invalid item type '" + string + "' specified.");
            }
            return null;
        }
        return new ItemTag(material);
    }

    public static boolean matches(String string) {
        if (string.startsWith("i@")) {
            return true;
        }
        class_2960 identifier = class_2960.method_12829(string);
        return identifier != null && (class_7923.field_41178.method_10250(identifier) || class_7923.field_41175.method_10250(identifier));
    }

    public static void register() {
        PropertyParser.registerPropertyTagHandlers(ItemTag.class, tagProcessor);
    }

    public static final ObjectTagProcessor<ItemTag> tagProcessor = new ObjectTagProcessor<>();

    @Override
    public ObjectTag getObjectAttribute(Attribute attribute) {
        return tagProcessor.getObjectAttribute(this, attribute);
    }

    @Override
    public void adjust(Mechanism mechanism) {
        tagProcessor.processMechanism(this, mechanism);
    }

    @Override
    public void applyProperty(Mechanism mechanism) {
        adjust(mechanism);
    }

    public class_1799 getStack() {
        return itemStack;
    }

    public String getName() {
        return Utilities.idToString(identifier);
    }

    @Override
    public String identify() {
        return "i@" + getName() + PropertyParser.getPropertiesString(this);
    }

    @Override
    public String identifySimple() {
        return "i@" + getName();
    }

    @Override
    public String debuggable() {
        return "<LG>i@<Y>" + getName() + PropertyParser.getPropertiesDebuggable(this);
    }

    @Override
    public String toString() {
        return identify();
    }

    @Override
    public boolean isUnique() {
        return false;
    }

    String prefix = "Item";

    @Override
    public String getPrefix() {
        return prefix;
    }

    @Override
    public ObjectTag setPrefix(String prefix) {
        this.prefix = prefix;
        return this;
    }
}
